/*
*
*           RaphaëlForm : HTML FORM elements adjustment for Raphaël         
*                                                                             
*                          version 0.9.dev (2014-08-20)                       
*  
*                    Copyright (C)  2013 - 2014  Jan Stransky                 
*  
*           Czech Technical University, Faculty of Civil Engineering,         
*       Department of Structural Mechanics, 166 29 Prague, Czech Republic     
*  
*  RaphaëlForm is free software: you can redistribute it and/or modify it
*  under the terms of the GNU Lesser General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or (at your
*  option) any later version.
*  
*  RaphaëlForm is distributed in the hope that it will be useful, but WITHOUT
*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
*  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
*  for more details.
*  
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/


/**
* @fileOverview HTML FORM elements adjustment for <a href="http://raphaeljs.com/">Raphaël</a> (svg javascript library). Requires <a href="http://raphaeljs.com/">raphael.js</a> loaded before loading this file. For more information see <a href="http://mech.fsv.cvut.cz/~stransky/software/raphaeltools/">project homepage</a>.
<br /><br/>RaphaëlForm is a free software distributed under <a href='http://www.gnu.org/licenses/gpl.html'>GNU GPL license</a>.
* @author <a href="http://mech.fsv.cvut.cz/~stransky/">Jan Stránský</a>
* @version Version 0.9.dev (2014-08-20)
*/



/** Global namespace object
* @namespace
*/
var RaphaelForm = {
};

// auxiliary object for repeated functions
RaphaelForm._ = {};

RaphaelForm._.attr = function(attrs,t) {
	if (!t) { t = this; }
	for (var a in attrs) {
		var val = attrs[a];
		if (a=='font-size') { a='fontSize'; }
		if (a=='x' || a=='y') {
			t.foreign.setAttribute(a,attrs[a]);
		} else {
			if (a=='fontSize' || a=='width' || a=='height') {
				if (typeof val == "number") {
					val = val + 'px';
				}
			}
			t.element.style[a] = val;
		}
		t.attrs[a] = val;
	}
	t.foreign.setAttribute('height',t.element.offsetHeight*1.2);
	t.foreign.setAttribute('width',t.element.offsetWidth);
	return t;
}

RaphaelForm._.onchange = function(f) {
	this.element.onchange = f;
	return this;
}

RaphaelForm._.onmouseover = function(f) {
	this.element.onmouseover = f;
	return this;
}

RaphaelForm._.onmouseout = function(f) {
	this.element.onmouseout = f;
	return this;
}

RaphaelForm._.foreign = function() {
	var foreign = document.createElementNS("http://www.w3.org/2000/svg",'foreignObject');
	foreign.x = '0';
	foreign.y = '0';
	foreign.style.padding = '0px';
	foreign.style.margin = '0px';
	return foreign;
}

RaphaelForm._.element = function(type) {
	var elem = document.createElementNS("http://www.w3.org/1999/xhtml",type);
	elem.style.padding = '0px';
	elem.style.margin = '0px';
	elem.setAttributeNS("http://www.w3.org/2000/xmlns/","xmlns","http://www.w3.org/1999/xhtml");
	return elem;
}

RaphaelForm._.appendToPaperCanvas = function(paper,foreign,element) {
	paper.canvas.appendChild(foreign);
	foreign.appendChild(element);
}








/** Label
* @class represents label
* @param {Raphael} paper Raphael Paper instance
* @param {string} text text
* @param {Object=} params object of additional parameters. Currently not used
* @property {Object} attrs attributes {x,y} of label
* @property {Element} element underlying html element
* @this RaphaelForm.Label
* @constructor
*/
RaphaelForm.Label = function(paper,text,params) {
	params = params || {};
	var foreign = this.foreign = RaphaelForm._.foreign()
	this.attrs = {
		x: 0,
		y: 0
	}
	var elem = this.element = RaphaelForm._.element('label');
	elem.innerHTML = text;
	RaphaelForm._.appendToPaperCanvas(paper,foreign,elem);
}

/** Sets attributes to receiver
* @function
* @param {Object} attrs attributes to be set
*/
RaphaelForm.Label.prototype.attr = RaphaelForm._.attr;

/** What to do on mouseover event
* @function
* @param f function to execute
*/
RaphaelForm.Label.prototype.onmouseover = RaphaelForm._.onmouseover;

/** What to do on mouseout event
* @function
* @param f function to execute
*/
RaphaelForm.Label.prototype.onmouseout = RaphaelForm._.onmouseout;

/** Creates new Label object. See {@link RaphaelForm.Label} for arguments
* @param {Paper} parent
* @param {string=} text
* @param {Object=} params
* @returns {RaphaelForm.Label} new Label object
*/
RaphaelForm.label = function(parent,text,params) {
	return new RaphaelForm.Label(parent,text,params);
}








/** Select
* @class represents select menu
* @param {Raphael} paper Raphael Paper instance
* @param {Array.<string>} options options of the menu
* @param {Object=} params object of additional parameters. params.values is used as values for options if defined, else options themself is used as values
* @property {Object} attrs attributes {x,y,width,height} of label
* @property {Element} element underlying html element
* @this RaphaelForm.Label
* @constructor
*/
RaphaelForm.Select = function(paper,options,params) {
	params = params || {};
	var values = params.values==undefined? options : params.values;
	if (options.length != values.length) { throw Error("options.length != params.values.length"); }
	var foreign = this.foreign = RaphaelForm._.foreign()
	this.attrs = {
		x: 0,
		y: 0,
		width: 100,
		height: 20
	}
	var elem = this.element = RaphaelForm._.element('select');
	elem.style.width = 100+'px';
	elem.style.height = 20+'px';
	var o;
	for (var i=0; i<options.length; i++) {
		o = document.createElement('option');
		o.value = values[i];
		o.innerHTML = options[i];
		o.setAttributeNS("http://www.w3.org/2000/xmlns/","xmlns","http://www.w3.org/1999/xhtml/");
		elem.appendChild(o)
	}
	RaphaelForm._.appendToPaperCanvas(paper,foreign,elem);
}

/** Sets attributes to receiver
* @function
* @param {Object} attrs attributes to be set
*/
RaphaelForm.Select.prototype.attr = RaphaelForm._.attr;

/** What to do on change event
* @function
* @param f function to execute
*/
RaphaelForm.Select.prototype.onchange = RaphaelForm._.onchange;

/** What to do on mouseover event
* @function
* @param f function to execute
*/
RaphaelForm.Select.prototype.onmouseover = RaphaelForm._.onmouseover;

/** What to do on mouseout event
* @function
* @param f function to execute
*/
RaphaelForm.Select.prototype.onmouseout = RaphaelForm._.onmouseout;

/** gets/sets selected index
* @param {number=} s selected index. If undefined, current selected index is returned. If defined, given selected index is set and Select object is returned.
*/
RaphaelForm.Select.prototype.selectedIndex = function(s) {
	if (s==undefined) { return this.element.selectedIndex; }
	this.element.selectedIndex = s;
	return this;
}

/** Returns array of receiver's values
*/
RaphaelForm.Select.prototype.values = function() {
	var ch = this.element.children;
	var ret = []
	for (i=0; i<ch.length; i++) {
		ret.push(ch[i].value);
	}
	return ret;
}

/** Returns current value
*/
RaphaelForm.Select.prototype.value = function() {
	return this.element.children[this.selectedIndex()].value;
}

/** Returns current option
*/
RaphaelForm.Select.prototype.option = function() {
	return this.element.children[this.selectedIndex()].innerHTML;
}

/** Returns array of receiver's options
*/
RaphaelForm.Select.prototype.options = function() {
	var ch = this.element.children;
	var ret = []
	for (i=0; i<ch.length; i++) {
		ret.push(ch[i].innerHTML);
	}
	return ret;
}

/** Creates new Select object. See {@link RaphaelForm.Select} for arguments
* @param {Paper} parent
* @param {Array.<string>} options
* @param {Object=} params
* @returns {RaphaelForm.Select} new Select object
*/
RaphaelForm.select = function(paper,options,params) {
	return new RaphaelForm.Select(paper,options,params);
}









/** InputText
* @class represents text input
* @param {Raphael} paper Raphael Paper instance
* @param {string=} text defeult value
* @param {Object=} params object of additional parameters. Currently not used
* @property {Object} attrs attributes {x,y,width,height} of label
* @property {Element} element underlying html element
* @this RaphaelForm.Label
* @constructor
*/
RaphaelForm.InputText = function(paper,text,params) {
	params = params || {};
	var foreign = this.foreign = RaphaelForm._.foreign()
	this.attrs = {
		x: 0,
		y: 0,
		width: 100,
		height: 20
	};
	var elem = this.element = RaphaelForm._.element('input');
	elem.type = 'text';
	elem.value = text;
	RaphaelForm._.appendToPaperCanvas(paper,foreign,elem);
}

/** Sets attributes to receiver
* @function
* @param {Object} attrs attributes to be set
*/
RaphaelForm.InputText.prototype.attr = RaphaelForm._.attr;

/** What to do on change event
* @function
* @param f function to execute
*/
RaphaelForm.InputText.prototype.onchange = RaphaelForm._.onchange;

/** What to do on mouseover event
* @function
* @param f function to execute
*/
RaphaelForm.InputText.prototype.onmouseover = RaphaelForm._.onmouseover;

/** What to do on mouseout event
* @function
* @param f function to execute
*/
RaphaelForm.InputText.prototype.onmouseout = RaphaelForm._.onmouseout;

/** gets/sets current value
* @param {string=} val value. If undefined, current value is returned. If defined, given value is set and InputText object is returned.
*/
RaphaelForm.InputText.prototype.value = function(val) {
	if (val === undefined) {
		return this.element.value;
	}
	this.element.value = val;
	return this;
}

/** Creates new InputText object. See {@link RaphaelForm.InputText} for arguments
* @param {Paper} parent
* @param {string} text
* @param {Object=} params
* @returns {RaphaelForm.InputText} new InputText object
*/
RaphaelForm.inputText = function(paper,text,params) {
	return new RaphaelForm.InputText(paper,text,params);
}
